package secret_test

import (
	"encoding/json"
	"fmt"
	"os"
	"os/exec"
	"path/filepath"
	"strings"
	"testing"
	"time"

	"git.eeqj.de/sneak/secret/internal/secret"
	"github.com/spf13/afero"
)

func TestPGPUnlockKeyWithRealFS(t *testing.T) {
	// Skip tests if gpg is not available
	if _, err := exec.LookPath("gpg"); err != nil {
		t.Skip("GPG not available, skipping PGP unlock key tests")
	}

	// Create a temporary directory for our tests
	tempDir, err := os.MkdirTemp("", "secret-pgp-test-")
	if err != nil {
		t.Fatalf("Failed to create temp dir: %v", err)
	}
	defer os.RemoveAll(tempDir) // Clean up after test

	// Create a temporary GNUPGHOME
	gnupgHomeDir := filepath.Join(tempDir, "gnupg")
	if err := os.MkdirAll(gnupgHomeDir, 0700); err != nil {
		t.Fatalf("Failed to create GNUPGHOME: %v", err)
	}

	// Save original GNUPGHOME
	origGnupgHome := os.Getenv("GNUPGHOME")

	// Set new GNUPGHOME
	os.Setenv("GNUPGHOME", gnupgHomeDir)

	// Clean up environment after test
	defer func() {
		if origGnupgHome != "" {
			os.Setenv("GNUPGHOME", origGnupgHome)
		} else {
			os.Unsetenv("GNUPGHOME")
		}
	}()

	// Create GPG batch file for key generation
	batchFile := filepath.Join(tempDir, "gen-key-batch")
	batchContent := `%echo Generating a test key
Key-Type: RSA
Key-Length: 2048
Name-Real: Test User
Name-Email: test@example.com
Expire-Date: 0
Passphrase: test123
%commit
%echo Key generation completed
`
	if err := os.WriteFile(batchFile, []byte(batchContent), 0600); err != nil {
		t.Fatalf("Failed to write batch file: %v", err)
	}

	// Generate GPG key
	t.Log("Generating GPG key...")
	cmd := exec.Command("gpg", "--batch", "--gen-key", batchFile)
	output, err := cmd.CombinedOutput()
	if err != nil {
		t.Fatalf("Failed to generate GPG key: %v\nOutput: %s", err, output)
	}
	t.Log("GPG key generated successfully")

	// Get the key ID
	cmd = exec.Command("gpg", "--list-secret-keys", "--with-colons")
	output, err = cmd.CombinedOutput()
	if err != nil {
		t.Fatalf("Failed to list GPG keys: %v\nOutput: %s", err, output)
	}

	// Parse output to get key ID
	var keyID string
	lines := strings.Split(string(output), "\n")
	for _, line := range lines {
		if strings.HasPrefix(line, "sec:") {
			fields := strings.Split(line, ":")
			if len(fields) >= 5 {
				keyID = fields[4]
				break
			}
		}
	}

	if keyID == "" {
		t.Fatalf("Failed to find GPG key ID in output: %s", output)
	}
	t.Logf("Generated GPG key ID: %s", keyID)

	// Export GNUPGHOME variable to ensure subprocesses inherit it
	err = os.Setenv("GNUPGHOME", gnupgHomeDir)
	if err != nil {
		t.Fatalf("Failed to set GNUPGHOME environment variable: %v", err)
	}

	// Use the real filesystem
	fs := afero.NewOsFs()

	// Test data
	testMnemonic := "abandon abandon abandon abandon abandon abandon abandon abandon abandon abandon abandon about"

	// Save original environment variable
	oldMnemonic := os.Getenv(secret.EnvMnemonic)
	oldGPGKeyID := os.Getenv(secret.EnvGPGKeyID)

	// Set test environment variables
	os.Setenv(secret.EnvMnemonic, testMnemonic)
	os.Setenv(secret.EnvGPGKeyID, keyID)

	// Clean up after test
	defer func() {
		if oldMnemonic != "" {
			os.Setenv(secret.EnvMnemonic, oldMnemonic)
		} else {
			os.Unsetenv(secret.EnvMnemonic)
		}

		if oldGPGKeyID != "" {
			os.Setenv(secret.EnvGPGKeyID, oldGPGKeyID)
		} else {
			os.Unsetenv(secret.EnvGPGKeyID)
		}
	}()

	// Create the directory structure for test
	keyDir := filepath.Join(tempDir, "unlock-key")
	if err := os.MkdirAll(keyDir, secret.DirPerms); err != nil {
		t.Fatalf("Failed to create key directory: %v", err)
	}

	// Set up test metadata
	metadata := secret.UnlockKeyMetadata{
		ID:        fmt.Sprintf("%s-pgp", keyID),
		Type:      "pgp",
		CreatedAt: time.Now(),
		Flags:     []string{"gpg", "encrypted"},
	}

	// We'll skip the CreatePGPUnlockKey test since it requires registered vault functions
	t.Run("CreatePGPUnlockKey", func(t *testing.T) {
		t.Skip("Skipping test that requires registered vault functions")
	})

	// Create a PGP unlock key for the remaining tests
	unlockKey := secret.NewPGPUnlockKey(fs, keyDir, metadata)

	// Test getting GPG key ID
	t.Run("GetGPGKeyID", func(t *testing.T) {
		// Create PGP metadata with GPG key ID
		type PGPUnlockKeyMetadata struct {
			secret.UnlockKeyMetadata
			GPGKeyID string `json:"gpg_key_id"`
		}

		pgpMetadata := PGPUnlockKeyMetadata{
			UnlockKeyMetadata: metadata,
			GPGKeyID:          keyID,
		}

		// Write metadata file
		metadataPath := filepath.Join(keyDir, "unlock-metadata.json")
		metadataBytes, err := json.MarshalIndent(pgpMetadata, "", "  ")
		if err != nil {
			t.Fatalf("Failed to marshal metadata: %v", err)
		}
		if err := afero.WriteFile(fs, metadataPath, metadataBytes, secret.FilePerms); err != nil {
			t.Fatalf("Failed to write metadata: %v", err)
		}

		// Get GPG key ID
		retrievedKeyID, err := unlockKey.GetGPGKeyID()
		if err != nil {
			t.Fatalf("Failed to get GPG key ID: %v", err)
		}

		// Verify key ID
		if retrievedKeyID != keyID {
			t.Errorf("Expected GPG key ID '%s', got '%s'", keyID, retrievedKeyID)
		}
	})

	// Test getting identity from PGP unlock key
	t.Run("GetIdentity", func(t *testing.T) {
		// For this test, we'll do a simplified version since GPG operations
		// can be tricky in automated tests
		t.Skip("Skipping GetIdentity test due to complex GPG operations in automated testing")
	})

	// Test removing the unlock key
	t.Run("RemoveUnlockKey", func(t *testing.T) {
		// Ensure key directory exists before removal
		keyExists, err := afero.DirExists(fs, keyDir)
		if err != nil {
			t.Fatalf("Failed to check if key directory exists: %v", err)
		}
		if !keyExists {
			t.Fatalf("Key directory does not exist: %s", keyDir)
		}

		// Remove unlock key
		err = unlockKey.Remove()
		if err != nil {
			t.Fatalf("Failed to remove unlock key: %v", err)
		}

		// Verify directory is gone
		keyExists, err = afero.DirExists(fs, keyDir)
		if err != nil {
			t.Fatalf("Failed to check if key directory exists: %v", err)
		}
		if keyExists {
			t.Errorf("Key directory still exists after removal: %s", keyDir)
		}
	})
}
